"""Canaveral entry point script."""
# canaveral_cli/__main__.py

from canaveral_cli import cli, __app_name__

def main() -> None:
    cli.app(prog_name=__app_name__)

if __name__ == "__main__":
    main()


"""This module provides the Cue Parser"""
# canaveral_cli/cue_parser.py

import re
# from langchain.chat_models import ChatOpenAI
# from langchain.prompts.chat import (
#     ChatPromptTemplate,
#     SystemMessagePromptTemplate,
#     HumanMessagePromptTemplate,
# )


# def parse_cue_llm(file: str) -> dict:
#     OPENAI_API_KEY = os.getenv('OPENAI_API_KEY')

#     chat = ChatOpenAI(client=OPENAI_API_KEY, temperature=0)

#     system_message_prompt = SystemMessagePromptTemplate.from_template(
#         """You will receive a CUE file for a configuration of a Kubernetes object. 
#         Parse the CUE file and return the name, description, type and parameters in the format of a json file.
#         If the file has "ui-hidden": "true" in the labels, return an empty json file.""")
#     human_message_prompt = HumanMessagePromptTemplate.from_template("{file}")

#     chat_prompt = ChatPromptTemplate.from_messages([system_message_prompt, human_message_prompt])

#     start = time.time()
#     response = chat(chat_prompt.format_prompt(file=file).to_messages())
#     print(f"Time to get response: {time.time() - start:.2f} seconds")
#     print(response.content)
#     return json.loads(response.content)

#     return {'type': 'component', 
#             'name': 'cronjob', 
#             'description': 'Describes cron jobs that run code or a script to completion.', 
#             'paramet': {'labels': 'string', 'annotations': 'int'}
#             }


def rem_quotation_marks(string: str) -> str:
    if string.startswith('"') and string.endswith('"'):
        return string[1:-1]
    return string

def parse_cue_hc(file: str) -> dict:
    # Go line by line and parse the file
    oam_file = {}
    current_parameter = {}
    state = 'name'
    for _, line in enumerate(file.splitlines()):
        # catch import and ignore inside of parens
        if line.startswith('import'):
            state = 'import'
        if state == 'import' and line.startswith(')'):
            state = 'name'
        if state == 'import':
            continue
        
        # catch name of type:
        if state == 'name' and line.find(': {') != -1:
            oam_file["name"] = line.split(':')[0]
            # if name has "", remove them
            oam_file["name"] = rem_quotation_marks(oam_file["name"])
            state = 'type-description-2'
            continue

        # description and type are at the same level, either one can appear first
        if (state == "type-description-2" or state == "type-description") and (line.find('description') != -1 or line.find('type') != -1):
            if line.find('description:') != -1:
                oam_file["description"] = line.split('description:')[1].strip()
                oam_file["description"] = rem_quotation_marks(oam_file["description"])
            if line.find('type:') != -1:
                oam_file["type"] = line.split('type:')[1].strip()
                oam_file["type"] = rem_quotation_marks(oam_file["type"])
            state = 'type-description' if state == 'type-description-2' else 'find-parameters'
            continue
        
        
        # catch parameters start
        if state == 'find-parameters' and line.startswith('\tparameter: '):
            oam_file["parameters"] = []
            if line.startswith('\tparameter: {'):
                state = 'parameter-list'
                continue
            else:
                # Specific case -> parameter: [string]: string | null
                current_parameter["name"] = None
                current_parameter["type"] = line.split('parameter: ')[1].strip()
                oam_file["parameters"].append(current_parameter)
                break
            
        # Ignore current parameter until new one
        if state == 'ignore-parameter' and (line.find('\n\n') != -1 or line.startswith('\t\t// +usage')):
            # Corner case in trait/expose.cue for line.startswith('\t\t// +usage')
            state = 'parameter-list'
            continue
        # Ignore current parameter, but if it is the last one, break
        if (state == 'ignore-parameter' or state == 'parameter-list') and line.startswith('\t}'):
            break
            # ignore parameter until blank line
        # skip blank lines between parameters
        if state == 'parameter-list' and line == '':
            continue

        # Parameters with comments. 
        # If in list and find comment, enter comment list
        # If already in comment list, keep here if find //
        if (state == 'parameter-list' or state == 'parameter-comment') and line.startswith('\t\t//'):
            if state == 'parameter-list':
                current_parameter = {}
            try:
                comment_type = line.split('=')[0].split('+')[1].strip()
                # print(f'comment_type: {comment_type}')
            except:
                # Corner case in policy/read-only.cue. Normal comment bellow informative comment
                if line.find('=') == -1:
                    continue
            else:
                match comment_type:
                    case 'usage':
                        if line.find('Deprecated') != -1:
                            state = 'ignore-parameter'
                            continue
                        else:
                            current_parameter["usage"] = line.split('=')[1].strip()
                    case 'short':
                        current_parameter["short"] = line.split('=')[1].strip()
                    case 'ignore':
                        state = 'ignore-parameter'
                        continue
                    case _:
                        raise Exception(f'Unknown comment type: {comment_type} in {oam_file["name"]}')
                state = 'parameter-comment'
                continue
            
        # if in comment and didn't find //, in parameter name
        # OR
        # if in parameter list and didn't find //, in parameter name
        if (state == 'parameter-comment') or (state == 'parameter-list' and re.search("^\t{2}[^\t\n:]+:[^\n]+", line)):
            name = line.split(':', 1)[0].strip()
            type_par = line.split(':', 1)[1].strip()
            #TODO Extract default value

            # If name has a ?, remove it and add current parameter["labels"] = false
            current_parameter['mandatory'] = True if name.find('?') == -1 else False
            name = name.replace('?', '') if name.find('?') != -1 else name

            current_parameter["name"] = name
            current_parameter["type"] = type_par
            oam_file["parameters"].append(current_parameter)
            # print(f'current_parameter appended: {current_parameter}')
            state = 'parameter-list'
            continue

        # Parameters can not have comments on top e.g.apply-deployment.cue
        

    return oam_file


"""This module provides funtions to update internal definitions"""
# canaveral_cli/definitions.py

import json
import os
from github import Github
from canaveral_cli import PACKAGEDIR

from canaveral_cli.cue_parser import parse_cue_hc


def clean_cue_file(cue) -> str:
    #* File can have too many tokens
    #* cut off #HealthProbe (and add closing bracket removed by cutting off #HealthProbe)
    hp_idx = cue.find("#HealthProbe: {")
    if hp_idx != -1:
        cue = cue[:hp_idx] + "}"
    #* remove parameters marked with // +usage=Deprecated
    
    depre_idx = cue.find("// +usage=Deprecated")
    while depre_idx != -1:
        # find nearest empty line before and after, delete all in between
        start_idx = cue.rfind("\n\n", 0, depre_idx)
        end_idx = cue.find("\n\n", depre_idx)
        cue = cue[:start_idx] + cue[end_idx:]
        depre_idx = cue.find("// +usage=Deprecated")

    return cue


def get_updated_type_data():
    GITHUB_API_KEY = os.getenv('GITHUB_API_KEY')
    g = Github(GITHUB_API_KEY)
    repo = g.get_repo("kubevela/kubevela")

    for element in ["component", "trait", "policy", "workflowstep"]:
        definitions = repo.get_contents(f'vela-templates/definitions/internal/{element}')
        if type(definitions) == list:
            for definition in definitions:
                # if definition.name != "task.cue":
                #     continue
                cue_file = definition.decoded_content.decode("utf-8")
                
                # If file has "ui-hidden": "true" in the labels, skip it
                if cue_file.find('"ui-hidden": "true"') != -1:
                    print(f"Skipping {element} definition: {definition.name}")
                    # If local file exists, delete it
                    if os.path.exists(PACKAGEDIR/f"data/oam_types/{element}/{definition.name[:-4]}.json"):
                        os.remove(PACKAGEDIR/f"data/oam_types/{element}/{definition.name[:-4]}.json")
                    continue
                
                # cue_file = clean_cue_file(cue_file)
                print(f'Parsing {element} definition: {definition.name}')
                parsed_dict = parse_cue_hc(cue_file)
                type_name = parsed_dict["name"]
                with open(PACKAGEDIR/f"data/oam_types/{element}/{type_name}.json", "w") as f:
                    f.write(json.dumps(parsed_dict))
                    f.close()
        else:
            raise Exception("No component definitions found in the Kubevela repository")
        


def get_ammounts_oam_file(oam_file: dict):
    """ Get the number of components, traits, policies, and workflowsteps in an OAM file """
    traits = 0
    for comp in oam_file["components"]:
        if "traits" in comp:
            traits += len(comp["traits"])
    
    policies = len(oam_file["policies"]) if "policies" in oam_file else 0
    workflow = len(oam_file["workflow"]) if "workflow" in oam_file else 0

    return len(oam_file["components"]), traits, policies, workflow


"""This module provides the merge_oam function"""
# canaveral_cli/merge_oam.py

def merge_oam(dev: dict, ops: dict):
    merged_yaml = {}
    # Keep header and components from dev
    merged_yaml["apiVersion"] = dev["apiVersion"]
    merged_yaml["kind"] = dev["kind"]
    merged_yaml["metadata"] = dev["metadata"]
    merged_yaml["spec"] = dev["spec"]

    # Add spec.policies and spec.workflow from ops to dev
    if "policies" in ops["spec"]:
        merged_yaml["spec"]["policies"] = ops["spec"]["policies"]
    if "workflow" in ops["spec"]:
        merged_yaml["spec"]["workflow"] = ops["spec"]["workflow"]

    # Add traits from ops to components with same name from dev, keep dev traits
    for dev_comp in merged_yaml["spec"]["components"]:
        for ops_comp in ops["spec"]["components"]:
            if dev_comp["name"] == ops_comp["name"]:
                if "traits" in dev_comp:
                    dev_comp["traits"] += ops_comp["traits"]
                else:
                    dev_comp["traits"] = ops_comp["traits"]

    return merged_yaml




"""This module provides the functions to create OAM files"""
# canaveral_cli/create_oam.py

from datetime import datetime
import json
import os
from pathlib import Path
import jinja2
import typer
from rich import print
from InquirerPy import inquirer
from InquirerPy.base.control import Choice
from InquirerPy.validator import EmptyInputValidator

from canaveral_cli import COMPONENT_TYPES, PACKAGEDIR, TRAIT_TYPES, POLICY_TYPES, WORKFLOWSTEP_TYPES

used_component_names = []
used_workflowstep_names = []

def get_type_data():
    for pair in [("component", COMPONENT_TYPES), ("trait", TRAIT_TYPES), ("policy", POLICY_TYPES), ("workflowstep", WORKFLOWSTEP_TYPES)]:
        for type_file in os.scandir(PACKAGEDIR/f"data/oam_types/{pair[0]}"):
            with open(type_file.path, "r") as f:
                pair[1].append(json.load(f))
                f.close()

    # move component with name webservice to first in list COMPONENT_TYPES
    for idx, component in enumerate(COMPONENT_TYPES):
        if component["name"] == "webservice":
            COMPONENT_TYPES.insert(0, COMPONENT_TYPES.pop(idx))
            break


def create_oam_file(oam_file_data: dict):
    environment = jinja2.Environment(
        loader=jinja2.FileSystemLoader(PACKAGEDIR/"data/templates/"), trim_blocks=True, lstrip_blocks=True)
    template = environment.get_template('vela_template.yaml.jinja')
    output_file = template.render(oam_file_data)
    # save file to form_data with current data and hour in file name
    with open(Path.home() / f"form_data/{datetime.now()}", "w") as f:
        f.write(output_file)
        f.close()
    with open("vela.yaml", "w") as f:
        f.write(output_file)
        f.close()
    print("\nThe created file is most likely [bold red]incomplete[/bold red].",
         "Please check it and fill the missing fields by consulting the [link=https://kubevela.io/docs/end-user/components/references]docs[/]", sep=os.linesep)
    typer.confirm(text="Confirm [Enter]", default=True, show_default=False)
    print("✅ vela.yaml created successfully! ")


def create_choice_list_component(components: list[dict]):
    choice_list = []
    for idx, component in enumerate(components):
        choice_list.append(Choice(name=component["name"], value=idx))
    return choice_list


def validator_name(name: str) -> bool:
    return True if len(name) > 0 and len(name) <= 63 and name[0].isalnum() and name[-1].isalnum() and all(
        [char.isalnum() or char in ["-", "_", "."] for char in name]) else False


def validator_component_name(name: str) -> bool:
    # The name segment is required and must be 63 characters or less,
    # beginning and ending with an alphanumeric character ([a-z0-9A-Z])
    # with dashes (-), underscores (_), dots (.), and alphanumerics between.
    # It cannot be empty or contain spaces.
    # It cannot be repeated between component names
    return True if len(name) > 0 and len(name) <= 63 and name[0].isalnum() and name[-1].isalnum() and all(
        [char.isalnum() or char in ["-", "_", "."] for char in name]) and name not in used_component_names else False


def validator_workflowstep_name(name: str) -> bool:
    # The name segment is required and must be 63 characters or less,
    # beginning and ending with an alphanumeric character ([a-z0-9A-Z])
    # with dashes (-), underscores (_), dots (.), and alphanumerics between.
    # It cannot be empty or contain spaces.
    # It cannot be repeated between workflowstep names
    return True if len(name) <= 63 and name[0].isalnum() and name[-1].isalnum() and all(
        [char.isalnum() or char in ["-", "_", "."] for char in name]) and name not in used_workflowstep_names else False


def oam_form():
    get_type_data()
    oam_file_data = {}
    used_component_names.clear()
    print("This tool will help you create a Vela file for your application.",
          "The file is made up of four sections: [green]components[/], [blue]traits[/], [yellow]policies[/] and [purple]workflowsteps[/].",
          "Information will be gathered through a series of questions, and details will be given throughout the process.", sep=os.linesep)
    typer.confirm(text="Confirm [Enter]", default=True, show_default=False)

    oam_file_data["app_name"] = inquirer.text(
        message="What is the name of the application? (without spaces)",
        validate=validator_name,
    ).execute()

    print("\n[bold green]COMPONENTS[/]",
          "Components are the building blocks of your application. They are the smallest unit of your application that can be independently deployed and managed.",
          "To define a component, we need a name, a type and a Docker image. Let's start with the first one.", sep=os.linesep)

    # COMPONENTS
    oam_file_data["components"] = []
    components_number = 1

    component_name = inquirer.text(
        message="What is the name of the component?",
        validate=validator_component_name,
    ).execute()

    print("\n[b]Types[/] are a contruct of OAM, they are used to define the type of component you are using.",
          "If in doubt, use [u]webservice[/u]. For something more specific, check the available component types in the [link=https://kubevela.io/docs/end-user/components/references]docs[/link].",
          sep=os.linesep)

    component_type_name = inquirer.fuzzy(
        message=f'What is the type of component "{component_name}"?',
        choices=[c['name'] for c in COMPONENT_TYPES],
    ).execute()
    component_image = inquirer.text(
        message=f'What is the Docker image of component "{component_name}"?',
        validate=EmptyInputValidator(),
    ).execute()

    component_type = next(
        (component for component in COMPONENT_TYPES if component["name"] == component_type_name), None)

    component = {"name": component_name,
                 "type": component_type,
                 "image": component_image,
                 }
    oam_file_data["components"].append(component)
    used_component_names.append(component_name)

    another_component = inquirer.confirm(
        message="Do you want to add another component?",
    ).execute()
    while another_component:
        components_number += 1
        component_name = inquirer.text(
            message="What is the name of the component?",
            validate=validator_component_name,
        ).execute()
        component_type_name = inquirer.fuzzy(
            message=f'What is the type of component "{component_name}"?',
            choices=[c['name'] for c in COMPONENT_TYPES],
        ).execute()
        component_image = inquirer.text(
            message=f'What is the Docker image of component "{component_name}"?',
            validate=EmptyInputValidator(),
        ).execute()

        component_type = next(
            (component for component in COMPONENT_TYPES if component["name"] == component_type_name), None)

        component = {"name": component_name,
                     "type": component_type,
                     "image": component_image,
                     }
        oam_file_data["components"].append(component)
        used_component_names.append(component_name)
        another_component = inquirer.confirm(
            message="Do you want to add another component?",
        ).execute()

    print("\n[bold blue]TRAITS[/] - Optional", 
          "Traits are used to add [u]extra functionality to your components[/]. They are optional, but you can add as many as you want.", 
          "Information about available traits can be found in the [link=https://kubevela.io/docs/end-user/traits/references]docs[/link].", sep=os.linesep)

    # TRAITS
    mess = "Do you want to add traits to any of the components?" if components_number > 1 else (
        f"Do you want to add traits to the component {component_name}?")
    traits = inquirer.confirm(
        message=mess,
    ).execute()

    while traits:
        if components_number == 1:
            current_comp_idx: int = 0
        else:
            print(f'There are {components_number} components')
            current_comp_idx: int = int(inquirer.rawlist(
                message="Which component do you want to add traits to?",
                choices=create_choice_list_component(
                    oam_file_data["components"]),
                default=None,
            ).execute())

        oam_file_data["components"][current_comp_idx]["traits"] = [
        ] if "traits" not in oam_file_data["components"][current_comp_idx] else oam_file_data["components"][current_comp_idx]["traits"]

        trait_type_name = inquirer.fuzzy(
            message="What is the type of the trait?",
            choices=[t['name'] for t in TRAIT_TYPES],
        ).execute()

        trait_type = next(
            (trait for trait in TRAIT_TYPES if trait["name"] == trait_type_name), None)

        trait = {"type": trait_type}
        oam_file_data["components"][current_comp_idx]["traits"].append(trait)

        another_trait = inquirer.confirm(
            message=f"Do you want to add another trait to component {oam_file_data['components'][current_comp_idx]['name']}?",
        ).execute()
        while another_trait:
            trait_type_name = inquirer.fuzzy(
                message="What is the type of the trait?",
                choices=[t['name'] for t in TRAIT_TYPES],
            ).execute()

            trait_type = next(
                (trait for trait in TRAIT_TYPES if trait["name"] == trait_type_name), None)

            trait = {"type": trait_type}
            oam_file_data["components"][current_comp_idx]["traits"].append(
                trait)
            another_trait = inquirer.confirm(
                message="Do you want to add another trait to the component?",
            ).execute()

        if components_number > 1:
            traits = inquirer.confirm(
                message="Do you want to add more traits to any of the components?",
            ).execute()
        else:
            traits = False

    # POLICIES
    print("\n[bold yellow]POLICIES[/] - Optional", 
          "Policies are used to add [u]extra functionality to your application[/]. They are optional, but you can add as many as you want.",
          "Information about available policies can be found in the [link=https://kubevela.io/docs/end-user/policies/references]docs[/link].", sep=os.linesep)

    policies = inquirer.confirm(
        message="Do you want to add policies to the application?",
    ).execute()

    if policies:
        oam_file_data["policies"] = []

        policy_type_name = inquirer.fuzzy(
            message="What is the type of the policy?",
            choices=[p['name'] for p in POLICY_TYPES],
        ).execute()
        policy_name = inquirer.text(
            message="What is the name of the policy?",
            validate=validator_name,
        ).execute()
        policy_type = next(
            (policy for policy in POLICY_TYPES if policy["name"] == policy_type_name), None)
        policy = {"name": policy_name,
                  "type": policy_type,
                  }
        oam_file_data["policies"].append(policy)

        another_policy = inquirer.confirm(
            message="Do you want to add another policy?",
        ).execute()
        while another_policy:
            policy_type_name = inquirer.fuzzy(
                message="What is the type of the policy?",
                choices=[p['name'] for p in POLICY_TYPES],
            ).execute()
            policy_name = inquirer.text(
                message="What is the name of the policy?",
                validate=validator_name,
            ).execute()
            policy_type = next(
                (policy for policy in POLICY_TYPES if policy["name"] == policy_type_name), None)
            policy = {"name": policy_name,
                      "type": policy_type,
                      }
            oam_file_data["policies"].append(policy)
            another_policy = inquirer.confirm(
                message="Do you want to add another policy?",
            ).execute()

    # WORKFLOW
    print("\n[bold purple]WORKFLOW[/] - Optional",
          "A workflow is used to [u]describe the steps to deploy your application[/]. It is optional, but you can define as many steps as you want.",
          "Information about available workflow steps can be found in the [link=https://kubevela.io/docs/end-user/workflow/built-in-workflow-defs]docs[/link].", sep=os.linesep)

    workflow = inquirer.confirm(
        message="Do you want to add a workflow to the application?",
    ).execute()

    if workflow:
        oam_file_data["workflow"] = []
        used_workflowstep_names.clear()
        print("The workflow is composed of a list of steps")
        workflow_step_type_name = inquirer.fuzzy(
            message="What is the first step?",
            choices=[w['name'] for w in WORKFLOWSTEP_TYPES],
        ).execute()
        workflow_step_name = inquirer.text(
            message="What is the name of the first step?",
            validate=validator_workflowstep_name,
        ).execute()
        workflow_step_type = next(
            (workflowstep for workflowstep in WORKFLOWSTEP_TYPES if workflowstep["name"] == workflow_step_type_name), None)
        workflow_step = {"name": workflow_step_name,
                         "type": workflow_step_type,
                         }
        oam_file_data["workflow"].append(workflow_step)

        another_step = inquirer.confirm(
            message="Do you want to add another step?",
        ).execute()
        while another_step:
            workflow_step_type_name = inquirer.fuzzy(
                message="What is the next step?",
                choices=[w['name'] for w in WORKFLOWSTEP_TYPES],
            ).execute()
            workflow_step_name = inquirer.text(
                message="What is the name of the next step?",
                validate=validator_workflowstep_name,
            ).execute()
            workflow_step_type = next(
                (workflowstep for workflowstep in WORKFLOWSTEP_TYPES if workflowstep["name"] == workflow_step_type_name), None)
            workflow_step = {"name": workflow_step_name,
                             "type": workflow_step_type,
                             }
            oam_file_data["workflow"].append(workflow_step)
            another_step = inquirer.confirm(
                message="Do you want to add another step?",
            ).execute()

    return oam_file_data



"""Top-level package for Canaveral"""
# canaveral_cli/__init__.py

import os
from dotenv import load_dotenv
from pathlib import Path
import logging

load_dotenv()

__app_name__ = "canaveral"
__version__ = "0.1.0"
PACKAGEDIR = Path(__file__).parent.absolute()

# check if form_data directory exists in home directory
if not os.path.isdir(Path.home() / "form_data"):
    os.mkdir(Path.home() / "form_data")

logging.basicConfig(filename=Path.home() / 'form_data/canaveral.log', level=logging.INFO, format='%(asctime)s %(levelname)s %(name)s %(message)s')

# COMPONENT_TYPES = ["webservice", "task", "cron-task", "daemon", "k8s-objects"]
# TRAIT_TYPES = ["affinity", "annotations", "command", "container-image", "cpuscaler", "env", "expose", "gateway", "hostalias", "hpa", "init-container", "json-merge-patch", "json-patch",
#                "k8s-update-strategy", "labels", "lifecycle", "nocalhost", "resource", "scaler", "service-account", "service-binding", "sidecar", "startup-probe", "storage", "topologyspreadconstraints"]
# POLICY_TYPES = ["apply-once", "garbage-collect", "health", "override",
#                 "read-only", "replication", "shared-resource", "take-over", "topology"]
# WORKFLOWSTEP_TYPES = ["addon-operation", "apply-app", "apply-component", "apply-deployment", "apply-object", "apply-terraform-config", "apply-terraform-provider", "build-push-image", "clean-jobs", "collect-service-endpoints", "create-config", "delete-config", "depends-on-app", "deploy",
#                       "deploy-cloud-resource", "export-data", "export2config", "export2secret", "generate-jdbc-connection", "list-config", "notification", "print-message-in-status", "read-app", "read-config", "read-object", "request", "share-cloud-resource", "step-group", "suspend", "vela-cli", "webhook"]
COMPONENT_TYPES = []
TRAIT_TYPES = []
POLICY_TYPES = []
WORKFLOWSTEP_TYPES = []



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['canaveral_cli']

package_data = \
{'': ['*'],
 'canaveral_cli': ['data/*',
                   'data/oam_types/component/*',
                   'data/oam_types/policy/*',
                   'data/oam_types/trait/*',
                   'data/oam_types/workflowstep/*',
                   'data/templates/*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'PyGithub>=1.58.2,<2.0.0',
 'PyYAML>=6.0,<7.0',
 'inquirerpy>=0.3.4,<0.4.0',
 'python-dotenv>=1.0.0,<2.0.0',
 'rich>=13.3.5,<14.0.0',
 'typer[all]>=0.9.0,<0.10.0']

entry_points = \
{'console_scripts': ['canaveral = canaveral_cli.cli:app']}

setup_kwargs = {
    'name': 'canaveral-cli',
    'version': '0.1.6',
    'description': "Helper CLI to interact with Devscope's internal platform codename Canaveral",
    'long_description': '# Canaveral CLI\n\nThis Project uses [Typer](https://typer.tiangolo.com/) for the CLI functionality and [InquirerPy](https://inquirerpy.readthedocs.io/en/latest/index.html#) for the parameter selection\n\n## Run it\nWorking with virtual envs in Python:\n- https://realpython.com/python-virtual-environments-a-primer\n- https://aaronlelevier.github.io/virtualenv-cheatsheet/\n\n```bash\n$ python3 -m venv venv --prompt="canaveral-env"\n$ source venv/bin/activate\n(canaveral-env) $ \n(canaveral-env) $ deactivate\n$ \n$ pip install -r requirements.txt\n$ OR\n$ python -m pip install -r requirements.txt\n```\n\n## How to contribuite\nProject layout to adhere to:\n- https://realpython.com/python-application-layouts/#command-line-application-layouts\n\n## Internal Notes\n\nDevscope Internal notes and documentation avaible at [Canaveral](https://devscope365.sharepoint.com/sites/academy/_layouts/OneNote.aspx?id=%2Fsites%2Facademy%2FSiteAssets%2FAcademy%20Notebook&wd=target%282023.one%7C1069493A-33E1-4F4C-8D77-28AA9AE54494%2FCanaveral%20CLI%7CF359B855-A8C4-4DB9-B416-CA0D700F2904%2F%29)\n',
    'author': 'André Gomes',
    'author_email': 'andre.gomes@devscope.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/DevScope/canaveral-cli',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



"""This module provides the Canaveral CLI"""
# canaveral_cli/cli.py

import logging
from typing import Optional
from pathlib import Path
import typer
import shutil
import yaml
from canaveral_cli import PACKAGEDIR, __app_name__, __version__
from canaveral_cli.definitions import get_updated_type_data
from canaveral_cli.helper import get_ammounts_oam_file
from canaveral_cli.merge_oam import merge_oam
from canaveral_cli.create_oam import oam_form, create_oam_file
from rich import print

app = typer.Typer()

def _version_callback(value: bool):
    if value:
        logging.info("version command")
        typer.echo(f"{__app_name__} v{__version__}")
        raise typer.Exit()

@app.command()
def definitions():
    """Retrieve OAM types from source and store them locally. Needed for creating OAM files"""
    get_updated_type_data()

@app.command()
def create():
    """Interactively Create a OAM file"""
    logging.info("Start create command")
    oam_form_data = oam_form()
    # with open(PACKAGEDIR/"data/raw_data.yaml", "w") as f:
    #     yaml.dump(oam_form_data, f)
    #     f.close()
    # return
    # oam_form_data = yaml.load(open(PACKAGEDIR/"data/raw_data.yaml"), Loader=yaml.FullLoader)
    
    oam_numbers = get_ammounts_oam_file(oam_form_data)

    # log the number of components, traits, policies, and workflowsteps
    logging.info(f"create - comp: {oam_numbers[0]}, traits: {oam_numbers[1]}, policies: {oam_numbers[2]}, workflowsteps: {oam_numbers[3]}")
    create_oam_file(oam_form_data)
    logging.info("End create command")

@app.command()
def default():
    """Create the default OAM file"""
    logging.info("default command")
    shutil.copyfile(PACKAGEDIR/"data/templates/vela_default.yaml", "vela.yaml")
    print("✅ vela.yaml created successfully! ")
    
@app.command()
def merge(dev: Path = typer.Argument(..., exists=True, file_okay=True, dir_okay=False, readable=True, resolve_path=True),
          ops: Path = typer.Argument(..., exists=True, file_okay=True, dir_okay=False, readable=True, resolve_path=True)):
    """Merge Dev OAM file with Operations OAM file"""
    logging.info("merge command")
    dev_yaml = yaml.load(dev.open(), Loader=yaml.FullLoader)
    ops_yaml = yaml.load(ops.open(), Loader=yaml.FullLoader)

    merged = merge_oam(dev_yaml, ops_yaml)
    
    with open("vela.yaml", "w") as f:
        yaml.dump(merged, f)
        f.close()

    #* Library hiyapyco doesn't work as expected, it doesn't merge nested lists
    # conf = hiyapyco.load([dev.absolute().as_posix(), ops.absolute().as_posix()], method=hiyapyco.METHOD_MERGE, interpolate=True)
    # print(type(conf))
    # with open("vela.yaml", "w") as f:
    #     f.write(hiyapyco.dump(conf, default_flow_style=False))
    #     f.close()


@app.callback()
def main(
    version: Optional[bool] = typer.Option(
        None,
        "--version",
        "-v",
        help="Show the application's version and exit",
        callback=_version_callback,
        is_eager=True,
    )
) -> None:
    return

